# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# OCaml implementation and API

import uuid
import binascii
import base64

class Plugin(object):

    '''
    Class: Plugin

    This class is an interface for plugins

    '''


    class OS(object):
        '''
        Class: OS

        This class encapsulates the comunication with an OS plugin using YAKS Evals
        '''
        def __init__(self,uuid, connector, node):
            self.uuid = uuid
            self.connector =  connector
            self.node =  node

        def call_os_plugin_function(self, fname, fparameters):
            '''
            Calls an Eval registered within the OS Plugin

            parameters
            ----------
            fname : string
                function name

            fparameters : dictionary
                parameters for the function

            returns
            ------
            whatever the fname function returns or raises a ValueError
            '''
            res = self.connector.loc.actual.exec_os_eval(
                self.node, fname, fparameters)
            if res.get('error'):
                raise ValueError('OS Eval returned {}'.format(res.get('error')))
                # return None
            return res.get('result')

        def dir_exists(self, dir_path):
            '''
            Checks if the given directory exists

            parameters
            ----------
            dir_path : string
                path to the directory

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('dir_exists',{'dir_path':dir_path})

        def create_dir(self, dir_path):
            '''
            Creates the given new directory

            parameters
            ----------
            dir_path : string
                path to the directory

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('create_dir',{'dir_path':dir_path})

        def download_file(self, url, file_path):
            '''
            Downloads the given file in the given path

            parameters
            ----------
            url : string
                url for the source file
            file_path : string
                path to destination file

            returns
            ------
            bool
            '''
            return self.call_os_plugin_function('download_file',{'url':url,'file_path':file_path})

        def execute_command(self, command, blocking=False, external=False):
            '''
            Executes a command on underlying os,

            parameters
            ---------

            command : string
                command to be executed

            blocking : bool
                true if the call has to block until the end of the command

            external : bool
                true if the command has to be executed in an external os shell

            returns
            ------
            string
            '''
            return self.call_os_plugin_function('execute_command',{'command':command,'blocking':blocking,'external':external})

        def remove_dir(self, dir_path):
            '''
            Removes the given directory

            parameters
            ----------
            dir_path : string
                path to the directory

            returns
            -------
            bool

            '''
            return self.call_os_plugin_function('remove_dir',{'dir_path':dir_path})

        def create_file(self, file_path):
            '''
            Creates the given new empty file

            parameters
            ----------
            file_path : string
                path to the file

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('create_file',{'file_path':file_path})

        def remove_file(self, file_path):
            '''
            Removes the given file

            parameters
            ----------
            file_path : string
                path to the directory

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('remove_file',{'file_path':file_path})

        def store_file(self, content, file_path, filename):
            '''
            Stores a file in local disk

            parameters
            ----------

            content : string
                file content

            file_path : string
                path where the content will stored

            filename : string
                name of the file

            returns
            -------
            bool
            '''
            content = binascii.hexlify(base64.b64encode(bytes(content,'utf-8'))).decode()
            return self.call_os_plugin_function('store_file',{'content':content,'file_path':file_path,'filename':filename})

        def read_file(self, file_path, root=False):
            '''
            Read the content from a file in the local disk,
            maybe can convert from windows dir separator to unix dir separator
            return the file content
            throw an exception if file not exits

            parameters
            ----------

            file_path : string
                path to file

            root : bool
                if true it will use sudo cat to read the file

            returns
            --------
            bytes
            '''
            return self.call_os_plugin_function('read_file',{'file_path':file_path,'root':root})

        def file_exists(self, file_path):
            '''
            Checks if the given file exists

            parameters
            ----------
            file_path : string
                path to the file

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('file_exists',{'file_path':file_path})

        def send_sig_int(self, pid):
            '''
            Sends a SigKill (Ctrl+C) to the given PID

            parameters
            ----------
            pid : int
                pid to be signaled

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('send_sig_int',{'pid':pid})

        def check_if_pid_exists(self, pid):
            '''
            Checks if a  given PID exists

            parameters
            ----------
            pid : int
                PID to be verified

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('check_if_pid_exists',{'pid':pid})

        def send_sig_kill(self, pid):
            '''
            Sends a SigKill (kill the process) to the given pid
            throw an exception if pid not exits

            parameters
            ----------
            pid : int
                pid to be signaled

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('send_sig_kill',{'pid':pid})

        def get_intf_type(self, name):
            '''
            Gets the inteface type for the given interface

            parameters
            ----------
            name : string
                name of the interface

            returns
            -------
            string
            '''
            return self.call_os_plugin_function('get_intf_type',{'name':name})

        def set_interface_unaviable(self, intf_name):
            '''
            Sets a given network device as unavailable

            paramters
            ---------
            intf_name : string
                name of the network device

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('set_interface_unaviable',{'intf_name':intf_name})

        def set_interface_available(self, intf_name):
            '''
            Sets a given network device as available

            paramters
            ---------
            intf_name : string
                name of the network device

            returns
            -------
            bool
            '''
            return self.call_os_plugin_function('set_interface_available',{'intf_name':intf_name})

        def get_network_informations(self):
            '''
            Gets information about node network interfaces

            returns
            -------
            list of dictionaties
                {
                    'intf_name':string,
                    'intf_mac_address':string,
                    'intf_speed': int,
                    'type':string,
                    'available':bool,
                    'default_gw':bool,
                    'intf_configuration':
                    {
                        'ipv4_address':string,
                        'ipv4_netmask':string,
                        'ipv4_gateway':string.
                        'ipv6_address':string,
                        'ipv6_netmask':string,
                        'ipv6_gateway':string.
                        'bus_address':string
                    }
                }

            '''
            return self.call_os_plugin_function('set_interface_available',{})

        def checksum(self, file_path):
            '''
            Calculates the SHA256 checksum for the given file

            parameters
            -----------
            file_path : string
                path to file

            returns
            -------
            string
            '''
            return self.call_os_plugin_function('checksum',{'file_path':file_path})

        def local_mgmt_address(self):
            '''
            Gets node management IP address

            returns
            -------
            string
            '''
            return self.call_os_plugin_function('local_mgmt_address',{})

    class NM(object):
        '''
        Class: NM

        This class encapsulates the comunication with an NM plugin using YAKS Evals
        '''

        def __init__(self, uuid, connector, node):
            self.uuid = uuid
            self.connector =  connector
            self.node =  node

        def call_nw_plugin_function(self, fname, fparameters):
            '''
            Calls an Eval registered within the NM Plugin

            parameters
            ----------
            fname : string
                function name

            fparameters : dictionary
                parameters for the function

            returns
            ------
            whatever the fname function returns or raises a ValueError
            '''
            res = self.connector.loc.actual.exec_nw_eval(
                self.node, self.uuid, fname, fparameters)
            if res.get('error'):
                raise ValueError('NM Eval returned {}'.format(res.get('error')))
                # return None
            return res.get('result')

        def create_virtual_interface(self,intf_id, descriptor):
            '''
            Creates a virtual network interface

            paramters
            ---------
            intf_id : string
                UUID of the virtual interface
            descriptor : dictionary
                descriptor of the virtual interface

            returns
            ---------
            dictionary
            '''
            return self.call_nw_plugin_function('create_virtual_interface',{'intf_id':intf_id,'descriptor':descriptor})

        def delete_virtual_interface(self, intf_id):
            '''
            Deletes the given virtual interface

            parameters
            ---------
            intf_id : string


            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('delete_virtual_interface',{'intf_id':intf_id})

        def create_virtual_bridge(self, name, uuid):
            '''
        Creates a virtual bridge

            parameters
            -----------
            name : string
                name of the virtual bridge to be created


            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('create_virtual_bridge',{'name':name,'uuid':uuid})

        def delete_virtual_bridge(self, br_uuid):
            '''
            Deletes a virtual bride

            parameters
            ----------
            br_uuid : string
                bridge UUID

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('delete_virtual_bridge',{'br_uuid':br_uuid})

        def create_bridges_if_not_exist(self, expected_bridges):
            '''
            Creates the bridges missing after checking the manifest file

            parameters
            ----------
            expected_bridges : string list
                names of expected bridges

            returns
            -------
            string list

            '''
            return self.call_nw_plugin_function('create_bridges_if_not_exist',{'expected_bridges':expected_bridges})

        def connect_interface_to_connection_point(self, intf_id, cp_id):
            '''
            Connects the given interace to the given connection point

            parameters
            ----------
            intf_id : string
                ID of the virtual interface
            cp_id : string
                UUID of the connection point

            returns
            -------
            dictionary
                {'int':string, 'ext':string}
            '''
            return self.call_nw_plugin_function('connect_interface_to_connection_point',{'intf_id':intf_id,'cp_id':cp_id})

        def disconnect_interface(self, intf_id):
            '''
            Disconnects the given interface

            parameters
            ----------
            intf_id : string
                ID of the virtual interace

            returns
            -------
            dictionary
                {'int':string, 'ext':string}
            '''
            return self.call_nw_plugin_function('disconnect_interface',{'intf_id':intf_id})

        def connect_cp_to_vnetwork(self, cp_id, vnet_id):
            '''
            Connects the given connection point to the given network

            parameters
            ----------
            cp_id : string
                UUID of the connection point
            vnet_id : string
                UUID of the virtual network

            returns
            -------
            dictionary
                {'int':string, 'ext':string}
            '''
            return self.call_nw_plugin_function('connect_cp_to_vnetwork',{'vnet_id':vnet_id,'cp_id':cp_id})

        def disconnect_cp(self, cp_id):
            '''
            Disconnects the given connection point

            parameters
            ----------
            cp_id : string
                UUID of connection point

            returns
            -------
            dictionary
                {'int':string, 'ext':string}
            '''
            return self.call_nw_plugin_function('disconnect_cp',{'cp_id':cp_id})

        def delete_port(self, cp_id):
            '''
            Deletes the given connection point

            parameters
            ----------
            cp_id : string
                UUID of the connection point

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('delete_port',{'cp_id':cp_id})

        def get_address(self, mac_address):
            '''
            Gets the IP address associated to the interface with the given MAC address

            parameters
            ----------
            mac_address : string
                the MAC address of the interface

            returns
            -------
            string
            '''
            return self.call_nw_plugin_function('get_address',{'mac_address':mac_address})

        def add_port_to_router(self, router_id, port_type, vnet_id=None, ip_address=None):
            '''
            Adds a port to the given virtual router

            parameters
            ----------
            router_id : string
                UUID of the virtual router
            port_type : string
                kind of the port to be added (INTERNAL, EXTERNAL)
            vnet_id : string
                eventual network to be connected
            ip_address : string
                eventual address for the new router port

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('add_router_port',{'router_id':router_id,'port_type':port_type,'vnet_id':vnet_id,'ip_address':ip_address})

        def remove_port_from_router(self, router_id, vnet_id):
            '''
            Removes a port from the given router

            ßparameters
            ----------
            router_id : string
                UUID of the virtual router
            vnet_id : string
                network to be disconnected

            returns
            -------
            dictionary

            '''
            return self.call_nw_plugin_function('remove_port_from_router',{'router_id':router_id,'vnet_id':vnet_id})

        def create_floating_ip(self):
            '''
            Creates a floating IP

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('create_floating_ip',{})

        def delete_floating_ip(self, ip_id):
            '''
            Deletes the given floating IP

            parameters
            ----------
            ip_id : string
                UUID of the floating IP

            returns
            -------
            dictionary

            '''
            return self.call_nw_plugin_function('delete_floating_ip',{'ip_id':ip_id})

        def assign_floating_ip(self, ip_id, cp_id):
            '''
            Assigns the given floating IP to the given conncetion point

            parameters
            ----------
            ip_id : string
                UUID of the floating IP
            cp_id : string
                UUID of the connection point

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('assign_floating_ip',{'ip_id':ip_id,'cp_id':cp_id})

        def remove_floating_ip(self, ip_id, cp_id):
            '''
            Retains the given floating IP from the given connection point

            parameters
            ----------
            ip_id : string
                UUID of the floating IP
            cp_id : string
                UUID of the connection point

            returns
            -------
            dictionary
            '''
            return self.call_nw_plugin_function('remove_floating_ip',{'ip_id':ip_id,'cp_id':cp_id})

        def get_overlay_face(self):
            '''
            Gets the configured interface for overlay networks

            returns
            -------
            string
            '''
            return self.call_nw_plugin_function('get_overlay_face',{})

        def get_vlan_face(self):
            '''
            Gets the configured interface for VLAN networks

            returns
            -------
            string
            '''
            return self.call_nw_plugin_function('get_vlan_face',{})


        def create_connection_point(self, descriptor):
            '''
            Creates a connection point from the given descriptor

            returns
            -------
            {'result': dictionary }
            '''

            return self.call_nw_plugin_function('create_port_agent',{'descriptor': descriptor})

        def remove_connection_point(self, cp_id):
            '''
            Deletes the given connection point

            returns
            -------
            {'result': dictionary }
            '''

            return self.call_nw_plugin_function('destroy_port_agent',{'cp_id':cp_id})

        def create_macvlan_interface(self, master_intf):
            '''
            Creates a macvlan interface over the given master interface

            returns
            -------
            {'result': string }
            '''

            return self.call_nw_plugin_function('create_macvlan_interface',{'master_intf': master_intf})

        def delete_macvlan_interface(self, intf_name, netns='1'):
            '''
            Deletes the given macvlan interface

            returns
            -------
            {'result': string }
            '''

            return self.call_nw_plugin_function('delete_macvlan_interface',{'intf_name':intf_name,'netns':netns})

        def create_network_namespace(self):
                '''
                Creates a new network namespace, and returns its name

                returns
                -------
                {'result':string}
                '''
                return self.call_nw_plugin_function('create_network_namespace',{})

        def delete_network_namespace(self, nsname):
            '''
            Deletes the given network namespace, and returns its name

            returns
            -------
            {'result':string}
            '''
            return self.call_nw_plugin_function('delete_network_namespace',{'nsname':nsname})


        def create_virtual_interface_in_namespace(self, internal_name, nsname):
            '''
            Creates a veth pair in the given network namespace, with the given name for the internal interface

            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('create_virtual_interface_in_namespace',{'internal_name':internal_name, 'nsname':nsname})

        def rename_virtual_interface_in_namespace(self, name, newname, nsname=None):
            '''
            Renames the given interface in the given namespace

            returns
            -------
            {'result': string }
            '''

            if nsname is None:
                nsname = '1'
            return self.call_nw_plugin_function('rename_virtual_interface_in_namespace',{'name':name,'newname':newname, 'nsname':nsname})


        def delete_virtual_interface_from_namespace(self, internal_name, nsname):
            '''
            Deletes the given interface from the the given network namespace

            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('delete_virtual_interface_from_namespace',{'internal_name':internal_name, 'nsname':nsname})


        def move_interface_in_namespace(self, intf_name, nsname=None):
            '''
            Moves the given interface into the the given network namespace, nsname is optional if none moves the interface to the default namespace
            returns
            -------
            {'result': dictionary }
            '''
            if nsname is None:
                nsname = '1'
            return self.call_nw_plugin_function('move_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname})

        def attach_interface_to_bridge(self, intf_name, br_name):
            '''
            Attaches the given interface to the given bridge
            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('attach_interface_to_bridge',{'intf_name':intf_name, 'br_name':br_name})

        def detach_interface_from_bridge(self, intf_name):
            '''
            Detaches the interface from the current connected bridge
            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('detach_interface_from_bridge',{'intf_name':intf_name})


        def assign_address_to_interface_in_namespace(self, intf_name, nsname, address=None):
            '''
            Assigns the given address to the given interface in the the given network namespace

            Address is in the form AAA.AAA.AAA.AAA/NM

            returns
            -------
            {'result': dictionary }
            '''
            if address is None:
                res = self.call_nw_plugin_function('assign_address_to_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname})
            else:
                res = self.call_nw_plugin_function('assign_address_to_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname, 'address':address})
            return res

        def assign_mac_address_to_interface_in_namespace(self, intf_name, nsname, address):
            '''
            Assigns the given address to the given interface in the the given network namespace

            Address are in the form AA:BB:CC:DD:EE:FF

            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('assign_mac_address_to_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname, 'address':address})

        def get_address_of_interface_in_namespace(self, intf_name, nsname):
            '''
            Retrieves the address to the given interface in the the given network namespace

            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('get_address_of_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname})

        def remove_address_from_interface_in_namespace(self, intf_name, nsname):
            '''
            Removes the address from the given interface in the the given network namespace
            returns
            -------
            {'result': dictionary }
            '''
            return self.call_nw_plugin_function('remove_address_from_interface_in_namespace',{'intf_name':intf_name, 'nsname':nsname})


    class Agent(object):
        '''
        Class: OS

        This class encapsulates the comunication with Agent using YAKS Evals
        '''
        def __init__(self, connector, node):
            self.connector =  connector
            self.node =  node

        def call_agent_function(self, fname, fparameters):
            '''
            Calls an Eval registered within the Agent

            parameters
            ----------
            fname : string
                function name

            fparameters : dictionary
                parameters for the function

            returns
            ------
            whatever the fname function returns or raises a ValueError
            '''
            res = self.connector.loc.actual.exec_agent_eval(
                self.node, fname, fparameters)
            if res.get('error') is not None:
                raise ValueError('Agent Eval returned {}'.format(res.get('error')))
            return res.get('result')

        def get_image_info(self, imageid):
            '''
            Gets information about the given image

            parameters
            ----------
            imageid : string
                UUID of the image

            returns
            -------
            dictionary
            '''
            return self.call_agent_function('get_image_info', {'image_uuid': imageid})

        def get_fdu_info(self, nodeid, fduid, instanceid):
            '''
            Gets information about the given FDU instance

            parameters
            ----------
            nodeid : string
                UUID of the node
            fduid : string
                UUID of the FDU
            instanceid : string
                UUID of the instance

            returns
            -------
            dictionary

            '''
            return self.call_agent_function('get_node_fdu_info', {'fdu_uuid':fduid,'instance_uuid':instanceid,'node_uuid':nodeid})

        def get_fdu_descriptor(self, fduid):
            '''
            Gets information about the given FDU

            parameters
            ----------
            fduid : string
                UUID of the FDU

            returns
            -------
            dictionary

            '''
            return self.call_agent_function('get_fdu_info', {'fdu_uuid':fduid})

        def get_network_info(self, uuid):
            '''
            Gets information about the given virtual network

            parameters
            ----------
            uuid : string
                UUID of the virtual network

            returns
            -------
            dictionary

            '''
            return self.call_agent_function('get_network_info',  {'uuid': uuid})

        def get_port_info(self, cp_uuid):
            '''
            Gets information about a given connection point

            parameters
            ----------
            cp_uuid : string
                UUID of the connection point

            returns
            -------
            dictionary
            '''
            return self.call_agent_function('get_port_info',  {'cp_uuid': cp_uuid})

        def get_node_mgmt_address(self, nodeid):
            '''
            Gets management IP address for the given node

            parameters
            ----------
            nodeid : string
                UUID of the node

            returns
            -------
            string
            '''
            return self.call_agent_function('get_node_mgmt_address', {'node_uuid': nodeid})


    def __init__(self, version, plugin_uuid=None):
        self.version = version
        self.connector = None
        self.node = None
        self.nm = None
        self.os = None
        self.agent = None
        if uuid is None:
            self.uuid = uuid.uuid4()
        else:
            self.uuid = plugin_uuid

    def get_nm_plugin(self):
        '''
        Retrives the network manager plugin from YAKS

        returns
        -------
        NM
        '''
        pls = self.connector.loc.actual.get_all_plugins(self.node)
        nms = [x for x in pls if x.get('type') == 'network']
        if len(nms) == 0:
            raise RuntimeError('No network_manager present in the node!!')
        nm = nms[0]
        self.nm = Plugin.NM(nm['uuid'], self.connector, self.node)
        return nm

    def get_os_plugin(self):
        '''
        Retrives the operating system plugin from YAKS

        returns
        -------
        OS
        '''
        pls = self.connector.loc.actual.get_all_plugins(self.node)
        os = [x for x in pls if x.get('type') == 'os']
        if len(os) == 0:
            raise RuntimeError('No os plugin present in the node!!')
        os = os[0]
        self.os = Plugin.OS(os['uuid'], self.connector, self.node)
        return os

    def get_agent(self):
        '''
        Retrives the agent from YAKS

        returns
        -------
        Agent
        '''
        self.agent = Plugin.Agent( self.connector, self.node)
        return self.agent

    def get_local_mgmt_address(self):
        '''
        Gets local management IP address

        returns
        -------
        string
        '''
        return self.os.local_mgmt_address()

    def get_version(self):
        '''
        Gets plugin version

        returns
        -------
        string
        '''
        return self.version




# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# Eclipse fog05 Python3 API


from fog05_sdk.yaks_connector import Yaks_Connector
from fog05_sdk.yaks_connector import Yaks_Constraint_Connector



# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# Atomic Entity Parsers and Python Class


import json
from fog05_sdk.types import infra_atomic_entity
from pyangbind.lib.serialise import pybindJSONDecoder
from pyangbind.lib.serialise import pybindJSONEncoder
from collections import OrderedDict
from fog05_sdk.interfaces.InfraFDU import InfraFDU
import copy

class AtomicEntityRecord(object):

    def __init__(self, data=None):
        '''

        Constructor for the FDU Descriptor
        :param data dictionary containing the FDU descriptor

        :return the FDU object

        '''

        self.ae = infra_atomic_entity.infra_atomic_entity()
        self.encoder = pybindJSONEncoder()
        self.atomic_entity_id = None
        self.uuid = None
        self.fdus = []
        self.internal_virtual_links = []
        self.connection_points = []
        self.depends_on = []
        if data is not None:

            def conv(x):
                x['properties'] = json.dumps(x['properties'])
                return x
            cdata = copy.deepcopy(data)
            xs = list(map(conv, cdata['connection_points']))
            cdata['connection_points'] = xs

            pybindJSONDecoder.load_ietf_json({'ae_record':cdata}, None, None, obj=self.ae, skip_unknown=True)
            self.enforce()

            self.atomic_entity_id = self.ae.ae_record.atomic_entity_id
            self.uuid = self.ae.ae_record.uuid
            self.fdus = [InfraFDU(x) for x in data.get('fdus')]
            self.internal_virtual_links = data.get('internal_virtual_links')
            self.connection_points = data.get('connection_points')
            self.depends_on = data.get('depends_on')



    def enforce(self):
        if self.ae.ae_record.atomic_entity_id == '':
            raise ValueError('AtomicEntityRecord.atomic_entity_id cannot be empty')

        if self.ae.ae_record.uuid == '':
            raise ValueError('AtomicEntityRecord.Name cannot be empty')

        if len (self.ae.ae_record.fdus) == 0:
            raise ValueError('AtomicEntityRecord.FDUS cannot be empty')

    def to_json(self):
        data = {
            'uuid': self.uuid,
            'atomic_entity_id': self.atomic_entity_id,
            'fdus': [x.to_json() for x in self.fdus],
            'internal_virtual_links': self.internal_virtual_links,
            'connection_points': self.connection_points,
            'depends_on': self.depends_on,
        }
        check_obj = infra_atomic_entity.infra_atomic_entity()

        def conv(x):
            x['properties'] = json.dumps(x['properties'])
            return x
        cdata = copy.deepcopy(data)
        xs = list(map(conv, cdata['connection_points']))
        cdata['connection_points'] = xs

        pybindJSONDecoder.load_ietf_json({'ae_record':cdata}, None, None, obj=check_obj, skip_unknown=True)
        return data

    def get_uuid(self):
        return self.uuid

    def set_uuid(self, uuid) :
        self.uuid = uuid

    def get_atomic_entity_id(self):
        return self.atomic_entity_id

    def set_atomic_entity_id(self, atomic_entity_id) :
        self.atomic_entity_id = atomic_entity_id

    def get_connection_points(self):
        return self.connection_points

    def set_connection_points(self, connection_points) :
        self.connection_points = connection_points

    def get_depends_on(self):
        return self.depends_on

    def set_depends_on(self, depends_on) :
        self.depends_on = depends_on


    def set_internal_virtual_links(self, virtual_links):
        self.internal_virtual_links = virtual_links

    def get_internal_virtual_links(self):
        return self.internal_virtual_links

    def set_fdus(self, fdus):
        self.fdus = fdus

    def get_fdus(self):
        return self.fdus

    def __str__(self):
        return "UUID : {} Atomic Entity ID: {}".format(self.uuid, self.atomic_entity_id)



# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# OCaml implementation and API


import uuid
import time
from fog05_sdk.interfaces.Plugin import Plugin
from fog05_sdk import Yaks_Connector
from fog05_sdk import Yaks_Connector
from fog05_sdk.DLogger import DLogger
from fog05_sdk.interfaces.InfraFDU import InfraFDU

class RuntimePluginFDU(Plugin):
    '''
    Class: RuntimePluginFDU

    This class is an interface for plugins that implements the FDU lifecycle management,
    and provides an abstraction layer
    for virtualisation capabilities
    '''

    def __init__(self,name, version, plugin_uuid, manifest):
        super(RuntimePluginFDU, self).__init__(version, plugin_uuid)
        self.pid = -1
        self.name = name
        loc = manifest.get('configuration').get('ylocator').split('/')[1]
        self.connector = Yaks_Connector(loc)
        self.logger = DLogger(debug_flag=True)
        self.node = manifest.get('configuration').get('nodeid')
        self.manifest = manifest
        self.configuration = manifest.get('configuration',{})

        self.current_fdus = {}

    def wait_destination_ready(self, fduid, instanceid, destinationid):
        '''
        Waits the destination node of a migration to be ready

        parameters
        ----------
        fduid : string
            UUID of the FDU
        instanceid : string
            UUID of the instance
        destinationid : string
            UUID of the destination node

        returns
        -------
        bool
        '''
        flag = False
        while not flag:
            try:
                res = self.agent.get_fdu_info(destinationid, fduid, instanceid)
                while res.get('status') != 'LAND':
                    time.sleep(0.250)
                    res = self.agent.get_fdu_info(destinationid, fduid, instanceid)
                flag = True
            except:
                pass
        return flag


    def wait_dependencies(self):
        '''
        Waits fot the Agent, OS and NM plugin to be up
        '''
        self.get_agent()
        os = None
        while os is None:
            try:
                os = self.get_os_plugin()
            except (RuntimeError, ValueError):
                time.sleep(1)
        nm = None
        while nm is None:
            try:
                nm = self.get_nm_plugin()
            except (RuntimeError, ValueError):
                time.sleep(1)
        return


    def write_fdu_error(self, fdu_uuid, instance_uuid, errno, errmsg):
        '''
        Writes the given error for the given FDU instance in YAKS

        parameters
        ----------
        fdu_uuid : string
            UUID of the FDU
        instance_uuid : string
            UUID of the instance
        errno : int
            Error number
        errmsg : string
            Error message
        '''
        record = self.connector.loc.actual.get_node_fdu(self.node, self.uuid, fdu_uuid, instance_uuid)
        if record is None:
            record = self.connector.loc.desired.get_node_fdu(self.node, self.uuid, fdu_uuid, instance_uuid)
        record = InfraFDU(record)
        record.set_status('ERROR')
        record.set_error_code(errno)
        record.set_error_msg('{}'.format(errmsg))
        self.connector.loc.actual.add_node_fdu(self.node, self.uuid, fdu_uuid, instance_uuid, record.to_json())

    def update_fdu_status(self, fdu_uuid, instance_uuid, status):
        '''
        Updates the status of the given FDU instance in YAKS

        parameters
        ----------
        fdu_uuid : string
            UUID of the FDU
        instance_uuid : string
            UUID of the instance
        status : string
            New status of the instance
        '''
        record = self.connector.loc.actual.get_node_fdu(self.node, self.uuid, fdu_uuid, instance_uuid)
        if record is None:
            record = self.connector.loc.desired.get_node_fdu(self.node, self.uuid, fdu_uuid, instance_uuid)
        record = InfraFDU(record)
        record.set_status(status)
        self.connector.loc.actual.add_node_fdu(self.node, self.uuid, fdu_uuid,instance_uuid, record.to_json())

    def get_local_instances(self, fdu_uuid):
        '''
        Gets all the local instances from YAKS

        returns
        -------
        string list
        '''
        return self.connector.loc.actual.get_node_fdu_instances(self.node, fdu_uuid)

    def start_runtime(self):
        '''
        Starts the plugin
        '''
        raise NotImplementedError('This is and interface!')

    def stop_runtime(self):
        '''
        Stops the plugin
        '''
        raise NotImplementedError('This is and interface!')

    def get_fdus(self):
        '''
        Gets all FDU and instances information

        returns
        -------
        dictionary
        '''
        raise NotImplementedError('This is and interface!')

    def define_fdu(self, fdu_record):
        '''
        Defines an FDU instance from the given record

        parameters
        ----------
        fdu_record : dictionary
            FDU instance record

        '''

        raise NotImplementedError('This is and interface!')

    def undefine_fdu(self, instance_uuid):
        '''
        Undefines the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def run_fdu(self, instance_uuid):
        '''
        Starts the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def stop_fdu(self, instance_uuid):
        ''''
        Stops the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def migrate_fdu(self, instance_uuid):
        '''
        Migrates the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''

        raise NotImplementedError('This is and interface!')

    def before_migrate_fdu_actions(self, instance_uuid, dst=False):
        '''
        Actions to be taken before migration of the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''

        raise NotImplementedError('This is and interface!')

    def after_migrate_fdu_actions(self, instance_uuid, dst=False):
        ''''
        Actions to be taken after the migration of the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''

        raise NotImplementedError('This is and interface!')

    def scale_fdu(self, instance_uuid):
        '''
        Scales the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def pause_fdu(self, instance_uuid):
        '''
        Pauses the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def resume_fdu(self, instance_uuid):
        '''
        Resumes the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def configure_fdu(self, instance_uuid):
        '''
        Configures the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def clean_fdu(self, instance_uuid):
        '''
        Cleans the given FDU instance

        parameters
        ----------
        instance_uuid : string
            UUID of the instance

        '''
        raise NotImplementedError('This is and interface!')

    def is_uuid(self, uuid_string):
        '''
        Verifies if the given string is a correct UUID4

        parameters
        ----------
        uuid_string : string
            the string to be verified

        returns
        -------
        bool
        '''
        try:
            val = uuid.UUID(uuid_string, version=4)
        except ValueError:
            return False
        return True


class FDUNotExistingException(Exception):
    def __init__(self, message, errors=0):

        super(FDUNotExistingException, self).__init__(message)
        self.errors = errors


class MigrationNotPossibleException(Exception):
    def __init__(self, message, errors=0):

        super(MigrationNotPossibleException, self).__init__(message)
        self.errors = errors


class MigrationNotAllowedException(Exception):
    def __init__(self, message, errors=0):

        super(MigrationNotAllowedException, self).__init__(message)
        self.errors = errors


class StateTransitionNotAllowedException(Exception):
    def __init__(self, message, errors=0):

        super(StateTransitionNotAllowedException, self).__init__(message)
        self.errors = errors



# -*- coding: utf-8 -*-
from operator import attrgetter
from pyangbind.lib.yangtypes import RestrictedPrecisionDecimalType
from pyangbind.lib.yangtypes import RestrictedClassType
from pyangbind.lib.yangtypes import TypedListType
from pyangbind.lib.yangtypes import YANGBool
from pyangbind.lib.yangtypes import YANGListType
from pyangbind.lib.yangtypes import YANGDynClass
from pyangbind.lib.yangtypes import ReferenceType
from pyangbind.lib.base import PybindBase
from collections import OrderedDict
from decimal import Decimal
from bitarray import bitarray
import six

# PY3 support of some PY2 keywords (needs improved)
if six.PY3:
  import builtins as __builtin__
  long = int
elif six.PY2:
  import __builtin__

class yc_atomic_entities_user_entity__entity_descriptor_atomic_entities(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /entity_descriptor/atomic_entities. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__id','__index',)

  _yang_name = 'atomic_entities'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__index = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="index", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return ['entity_descriptor', 'atomic_entities']

  def _get_id(self):
    """
    Getter method for id, mapped from YANG variable /entity_descriptor/atomic_entities/id (string)
    """
    return self.__id
      
  def _set_id(self, v, load=False):
    """
    Setter method for id, mapped from YANG variable /entity_descriptor/atomic_entities/id (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_id is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_id() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """id must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__id = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_id(self):
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_index(self):
    """
    Getter method for index, mapped from YANG variable /entity_descriptor/atomic_entities/index (int64)
    """
    return self.__index
      
  def _set_index(self, v, load=False):
    """
    Setter method for index, mapped from YANG variable /entity_descriptor/atomic_entities/index (int64)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_index is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_index() directly.
    """
    parent = getattr(self, "_parent", None)
    if parent is not None and load is False:
      raise AttributeError("Cannot set keys directly when" +
                             " within an instantiated list")

    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="index", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """index must be of a type compatible with int64""",
          'defined-type': "int64",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="index", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)""",
        })

    self.__index = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_index(self):
    self.__index = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="index", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)

  id = __builtin__.property(_get_id, _set_id)
  index = __builtin__.property(_get_index, _set_index)


  _pyangbind_elements = OrderedDict([('id', id), ('index', index), ])


class yc_cps_user_entity__entity_descriptor_virtual_links_cps(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /entity_descriptor/virtual_links/cps. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__component_id_ref','__component_index_ref','__cp_id','__has_floating_ip',)

  _yang_name = 'cps'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__component_id_ref = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="component_id_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__component_index_ref = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="component_index_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    self.__cp_id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="cp_id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__has_floating_ip = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="has_floating_ip", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return ['entity_descriptor', 'virtual_links', 'cps']

  def _get_component_id_ref(self):
    """
    Getter method for component_id_ref, mapped from YANG variable /entity_descriptor/virtual_links/cps/component_id_ref (string)
    """
    return self.__component_id_ref
      
  def _set_component_id_ref(self, v, load=False):
    """
    Setter method for component_id_ref, mapped from YANG variable /entity_descriptor/virtual_links/cps/component_id_ref (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_component_id_ref is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_component_id_ref() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="component_id_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """component_id_ref must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="component_id_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__component_id_ref = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_component_id_ref(self):
    self.__component_id_ref = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="component_id_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_component_index_ref(self):
    """
    Getter method for component_index_ref, mapped from YANG variable /entity_descriptor/virtual_links/cps/component_index_ref (int64)
    """
    return self.__component_index_ref
      
  def _set_component_index_ref(self, v, load=False):
    """
    Setter method for component_index_ref, mapped from YANG variable /entity_descriptor/virtual_links/cps/component_index_ref (int64)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_component_index_ref is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_component_index_ref() directly.
    """
    parent = getattr(self, "_parent", None)
    if parent is not None and load is False:
      raise AttributeError("Cannot set keys directly when" +
                             " within an instantiated list")

    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="component_index_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """component_index_ref must be of a type compatible with int64""",
          'defined-type': "int64",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="component_index_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)""",
        })

    self.__component_index_ref = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_component_index_ref(self):
    self.__component_index_ref = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="component_index_ref", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)


  def _get_cp_id(self):
    """
    Getter method for cp_id, mapped from YANG variable /entity_descriptor/virtual_links/cps/cp_id (string)
    """
    return self.__cp_id
      
  def _set_cp_id(self, v, load=False):
    """
    Setter method for cp_id, mapped from YANG variable /entity_descriptor/virtual_links/cps/cp_id (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_cp_id is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_cp_id() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="cp_id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """cp_id must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="cp_id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__cp_id = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_cp_id(self):
    self.__cp_id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="cp_id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_has_floating_ip(self):
    """
    Getter method for has_floating_ip, mapped from YANG variable /entity_descriptor/virtual_links/cps/has_floating_ip (boolean)
    """
    return self.__has_floating_ip
      
  def _set_has_floating_ip(self, v, load=False):
    """
    Setter method for has_floating_ip, mapped from YANG variable /entity_descriptor/virtual_links/cps/has_floating_ip (boolean)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_has_floating_ip is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_has_floating_ip() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGBool, is_leaf=True, yang_name="has_floating_ip", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """has_floating_ip must be of a type compatible with boolean""",
          'defined-type': "boolean",
          'generated-type': """YANGDynClass(base=YANGBool, is_leaf=True, yang_name="has_floating_ip", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)""",
        })

    self.__has_floating_ip = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_has_floating_ip(self):
    self.__has_floating_ip = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="has_floating_ip", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)

  component_id_ref = __builtin__.property(_get_component_id_ref, _set_component_id_ref)
  component_index_ref = __builtin__.property(_get_component_index_ref, _set_component_index_ref)
  cp_id = __builtin__.property(_get_cp_id, _set_cp_id)
  has_floating_ip = __builtin__.property(_get_has_floating_ip, _set_has_floating_ip)


  _pyangbind_elements = OrderedDict([('component_id_ref', component_id_ref), ('component_index_ref', component_index_ref), ('cp_id', cp_id), ('has_floating_ip', has_floating_ip), ])


class yc_ip_configuration_user_entity__entity_descriptor_virtual_links_ip_configuration(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /entity_descriptor/virtual_links/ip_configuration. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__ip_version','__subnet','__gateway','__checksum','__dhcp_enable','__dhcp_range','__dns',)

  _yang_name = 'ip_configuration'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__ip_version = YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'IPV4': {}, 'IPV6': {}},), is_leaf=True, yang_name="ip_version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='ip_kind', is_config=True)
    self.__subnet = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="subnet", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__gateway = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="gateway", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__checksum = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="checksum", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__dhcp_enable = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="dhcp_enable", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)
    self.__dhcp_range = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="dhcp_range", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__dns = YANGDynClass(unique=True, base=TypedListType(allowed_type=six.text_type), is_leaf=False, yang_name="dns", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return ['entity_descriptor', 'virtual_links', 'ip_configuration']

  def _get_ip_version(self):
    """
    Getter method for ip_version, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/ip_version (ip_kind)
    """
    return self.__ip_version
      
  def _set_ip_version(self, v, load=False):
    """
    Setter method for ip_version, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/ip_version (ip_kind)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_ip_version is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_ip_version() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'IPV4': {}, 'IPV6': {}},), is_leaf=True, yang_name="ip_version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='ip_kind', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """ip_version must be of a type compatible with ip_kind""",
          'defined-type': "user_entity:ip_kind",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'IPV4': {}, 'IPV6': {}},), is_leaf=True, yang_name="ip_version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='ip_kind', is_config=True)""",
        })

    self.__ip_version = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_ip_version(self):
    self.__ip_version = YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'IPV4': {}, 'IPV6': {}},), is_leaf=True, yang_name="ip_version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='ip_kind', is_config=True)


  def _get_subnet(self):
    """
    Getter method for subnet, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/subnet (string)
    """
    return self.__subnet
      
  def _set_subnet(self, v, load=False):
    """
    Setter method for subnet, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/subnet (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_subnet is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_subnet() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="subnet", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """subnet must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="subnet", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__subnet = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_subnet(self):
    self.__subnet = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="subnet", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_gateway(self):
    """
    Getter method for gateway, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/gateway (string)
    """
    return self.__gateway
      
  def _set_gateway(self, v, load=False):
    """
    Setter method for gateway, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/gateway (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_gateway is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_gateway() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="gateway", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """gateway must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="gateway", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__gateway = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_gateway(self):
    self.__gateway = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="gateway", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_checksum(self):
    """
    Getter method for checksum, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/checksum (string)

    YANG Description: SHA1 checksum of the image file
    """
    return self.__checksum
      
  def _set_checksum(self, v, load=False):
    """
    Setter method for checksum, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/checksum (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_checksum is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_checksum() directly.

    YANG Description: SHA1 checksum of the image file
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="checksum", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """checksum must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="checksum", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__checksum = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_checksum(self):
    self.__checksum = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="checksum", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_dhcp_enable(self):
    """
    Getter method for dhcp_enable, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dhcp_enable (boolean)
    """
    return self.__dhcp_enable
      
  def _set_dhcp_enable(self, v, load=False):
    """
    Setter method for dhcp_enable, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dhcp_enable (boolean)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_dhcp_enable is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_dhcp_enable() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGBool, is_leaf=True, yang_name="dhcp_enable", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """dhcp_enable must be of a type compatible with boolean""",
          'defined-type': "boolean",
          'generated-type': """YANGDynClass(base=YANGBool, is_leaf=True, yang_name="dhcp_enable", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)""",
        })

    self.__dhcp_enable = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_dhcp_enable(self):
    self.__dhcp_enable = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="dhcp_enable", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)


  def _get_dhcp_range(self):
    """
    Getter method for dhcp_range, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dhcp_range (string)
    """
    return self.__dhcp_range
      
  def _set_dhcp_range(self, v, load=False):
    """
    Setter method for dhcp_range, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dhcp_range (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_dhcp_range is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_dhcp_range() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="dhcp_range", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """dhcp_range must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="dhcp_range", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__dhcp_range = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_dhcp_range(self):
    self.__dhcp_range = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="dhcp_range", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_dns(self):
    """
    Getter method for dns, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dns (string)
    """
    return self.__dns
      
  def _set_dns(self, v, load=False):
    """
    Setter method for dns, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration/dns (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_dns is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_dns() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,unique=True, base=TypedListType(allowed_type=six.text_type), is_leaf=False, yang_name="dns", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """dns must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(unique=True, base=TypedListType(allowed_type=six.text_type), is_leaf=False, yang_name="dns", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__dns = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_dns(self):
    self.__dns = YANGDynClass(unique=True, base=TypedListType(allowed_type=six.text_type), is_leaf=False, yang_name="dns", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)

  ip_version = __builtin__.property(_get_ip_version, _set_ip_version)
  subnet = __builtin__.property(_get_subnet, _set_subnet)
  gateway = __builtin__.property(_get_gateway, _set_gateway)
  checksum = __builtin__.property(_get_checksum, _set_checksum)
  dhcp_enable = __builtin__.property(_get_dhcp_enable, _set_dhcp_enable)
  dhcp_range = __builtin__.property(_get_dhcp_range, _set_dhcp_range)
  dns = __builtin__.property(_get_dns, _set_dns)


  _pyangbind_elements = OrderedDict([('ip_version', ip_version), ('subnet', subnet), ('gateway', gateway), ('checksum', checksum), ('dhcp_enable', dhcp_enable), ('dhcp_range', dhcp_range), ('dns', dns), ])


class yc_virtual_links_user_entity__entity_descriptor_virtual_links(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /entity_descriptor/virtual_links. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__id','__name','__is_mgmt','__short_name','__description','__vl_type','__root_bandwidth','__leaf_bandwidth','__cps','__ip_configuration',)

  _yang_name = 'virtual_links'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__is_mgmt = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="is_mgmt", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)
    self.__short_name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="short_name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__description = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__vl_type = YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'ELINE': {}, 'ELAN': {}},), is_leaf=True, yang_name="vl_type", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='vl_kind', is_config=True)
    self.__root_bandwidth = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="root_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    self.__leaf_bandwidth = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="leaf_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    self.__cps = YANGDynClass(base=YANGListType("component_index_ref",yc_cps_user_entity__entity_descriptor_virtual_links_cps, yang_name="cps", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='component_index_ref', extensions=None), is_container='list', yang_name="cps", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)
    self.__ip_configuration = YANGDynClass(base=yc_ip_configuration_user_entity__entity_descriptor_virtual_links_ip_configuration, is_container='container', yang_name="ip_configuration", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return ['entity_descriptor', 'virtual_links']

  def _get_id(self):
    """
    Getter method for id, mapped from YANG variable /entity_descriptor/virtual_links/id (string)
    """
    return self.__id
      
  def _set_id(self, v, load=False):
    """
    Setter method for id, mapped from YANG variable /entity_descriptor/virtual_links/id (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_id is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_id() directly.
    """
    parent = getattr(self, "_parent", None)
    if parent is not None and load is False:
      raise AttributeError("Cannot set keys directly when" +
                             " within an instantiated list")

    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """id must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__id = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_id(self):
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, is_keyval=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_name(self):
    """
    Getter method for name, mapped from YANG variable /entity_descriptor/virtual_links/name (string)
    """
    return self.__name
      
  def _set_name(self, v, load=False):
    """
    Setter method for name, mapped from YANG variable /entity_descriptor/virtual_links/name (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_name is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_name() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """name must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__name = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_name(self):
    self.__name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_is_mgmt(self):
    """
    Getter method for is_mgmt, mapped from YANG variable /entity_descriptor/virtual_links/is_mgmt (boolean)
    """
    return self.__is_mgmt
      
  def _set_is_mgmt(self, v, load=False):
    """
    Setter method for is_mgmt, mapped from YANG variable /entity_descriptor/virtual_links/is_mgmt (boolean)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_is_mgmt is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_is_mgmt() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGBool, is_leaf=True, yang_name="is_mgmt", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """is_mgmt must be of a type compatible with boolean""",
          'defined-type': "boolean",
          'generated-type': """YANGDynClass(base=YANGBool, is_leaf=True, yang_name="is_mgmt", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)""",
        })

    self.__is_mgmt = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_is_mgmt(self):
    self.__is_mgmt = YANGDynClass(base=YANGBool, is_leaf=True, yang_name="is_mgmt", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='boolean', is_config=True)


  def _get_short_name(self):
    """
    Getter method for short_name, mapped from YANG variable /entity_descriptor/virtual_links/short_name (string)
    """
    return self.__short_name
      
  def _set_short_name(self, v, load=False):
    """
    Setter method for short_name, mapped from YANG variable /entity_descriptor/virtual_links/short_name (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_short_name is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_short_name() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="short_name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """short_name must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="short_name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__short_name = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_short_name(self):
    self.__short_name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="short_name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_description(self):
    """
    Getter method for description, mapped from YANG variable /entity_descriptor/virtual_links/description (string)
    """
    return self.__description
      
  def _set_description(self, v, load=False):
    """
    Setter method for description, mapped from YANG variable /entity_descriptor/virtual_links/description (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_description is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_description() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """description must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__description = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_description(self):
    self.__description = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_vl_type(self):
    """
    Getter method for vl_type, mapped from YANG variable /entity_descriptor/virtual_links/vl_type (vl_kind)
    """
    return self.__vl_type
      
  def _set_vl_type(self, v, load=False):
    """
    Setter method for vl_type, mapped from YANG variable /entity_descriptor/virtual_links/vl_type (vl_kind)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_vl_type is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_vl_type() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'ELINE': {}, 'ELAN': {}},), is_leaf=True, yang_name="vl_type", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='vl_kind', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """vl_type must be of a type compatible with vl_kind""",
          'defined-type': "user_entity:vl_kind",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'ELINE': {}, 'ELAN': {}},), is_leaf=True, yang_name="vl_type", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='vl_kind', is_config=True)""",
        })

    self.__vl_type = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_vl_type(self):
    self.__vl_type = YANGDynClass(base=RestrictedClassType(base_type=six.text_type,                                     restriction_type="dict_key",                                     restriction_arg={'ELINE': {}, 'ELAN': {}},), is_leaf=True, yang_name="vl_type", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='vl_kind', is_config=True)


  def _get_root_bandwidth(self):
    """
    Getter method for root_bandwidth, mapped from YANG variable /entity_descriptor/virtual_links/root_bandwidth (int64)
    """
    return self.__root_bandwidth
      
  def _set_root_bandwidth(self, v, load=False):
    """
    Setter method for root_bandwidth, mapped from YANG variable /entity_descriptor/virtual_links/root_bandwidth (int64)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_root_bandwidth is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_root_bandwidth() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="root_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """root_bandwidth must be of a type compatible with int64""",
          'defined-type': "int64",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="root_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)""",
        })

    self.__root_bandwidth = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_root_bandwidth(self):
    self.__root_bandwidth = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="root_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)


  def _get_leaf_bandwidth(self):
    """
    Getter method for leaf_bandwidth, mapped from YANG variable /entity_descriptor/virtual_links/leaf_bandwidth (int64)
    """
    return self.__leaf_bandwidth
      
  def _set_leaf_bandwidth(self, v, load=False):
    """
    Setter method for leaf_bandwidth, mapped from YANG variable /entity_descriptor/virtual_links/leaf_bandwidth (int64)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_leaf_bandwidth is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_leaf_bandwidth() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="leaf_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """leaf_bandwidth must be of a type compatible with int64""",
          'defined-type': "int64",
          'generated-type': """YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="leaf_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)""",
        })

    self.__leaf_bandwidth = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_leaf_bandwidth(self):
    self.__leaf_bandwidth = YANGDynClass(base=RestrictedClassType(base_type=long, restriction_dict={'range': ['-9223372036854775808..9223372036854775807']}, int_size=64), is_leaf=True, yang_name="leaf_bandwidth", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='int64', is_config=True)


  def _get_cps(self):
    """
    Getter method for cps, mapped from YANG variable /entity_descriptor/virtual_links/cps (list)
    """
    return self.__cps
      
  def _set_cps(self, v, load=False):
    """
    Setter method for cps, mapped from YANG variable /entity_descriptor/virtual_links/cps (list)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_cps is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_cps() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGListType("component_index_ref",yc_cps_user_entity__entity_descriptor_virtual_links_cps, yang_name="cps", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='component_index_ref', extensions=None), is_container='list', yang_name="cps", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """cps must be of a type compatible with list""",
          'defined-type': "list",
          'generated-type': """YANGDynClass(base=YANGListType("component_index_ref",yc_cps_user_entity__entity_descriptor_virtual_links_cps, yang_name="cps", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='component_index_ref', extensions=None), is_container='list', yang_name="cps", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)""",
        })

    self.__cps = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_cps(self):
    self.__cps = YANGDynClass(base=YANGListType("component_index_ref",yc_cps_user_entity__entity_descriptor_virtual_links_cps, yang_name="cps", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='component_index_ref', extensions=None), is_container='list', yang_name="cps", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)


  def _get_ip_configuration(self):
    """
    Getter method for ip_configuration, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration (container)
    """
    return self.__ip_configuration
      
  def _set_ip_configuration(self, v, load=False):
    """
    Setter method for ip_configuration, mapped from YANG variable /entity_descriptor/virtual_links/ip_configuration (container)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_ip_configuration is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_ip_configuration() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=yc_ip_configuration_user_entity__entity_descriptor_virtual_links_ip_configuration, is_container='container', yang_name="ip_configuration", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """ip_configuration must be of a type compatible with container""",
          'defined-type': "container",
          'generated-type': """YANGDynClass(base=yc_ip_configuration_user_entity__entity_descriptor_virtual_links_ip_configuration, is_container='container', yang_name="ip_configuration", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)""",
        })

    self.__ip_configuration = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_ip_configuration(self):
    self.__ip_configuration = YANGDynClass(base=yc_ip_configuration_user_entity__entity_descriptor_virtual_links_ip_configuration, is_container='container', yang_name="ip_configuration", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)

  id = __builtin__.property(_get_id, _set_id)
  name = __builtin__.property(_get_name, _set_name)
  is_mgmt = __builtin__.property(_get_is_mgmt, _set_is_mgmt)
  short_name = __builtin__.property(_get_short_name, _set_short_name)
  description = __builtin__.property(_get_description, _set_description)
  vl_type = __builtin__.property(_get_vl_type, _set_vl_type)
  root_bandwidth = __builtin__.property(_get_root_bandwidth, _set_root_bandwidth)
  leaf_bandwidth = __builtin__.property(_get_leaf_bandwidth, _set_leaf_bandwidth)
  cps = __builtin__.property(_get_cps, _set_cps)
  ip_configuration = __builtin__.property(_get_ip_configuration, _set_ip_configuration)


  _pyangbind_elements = OrderedDict([('id', id), ('name', name), ('is_mgmt', is_mgmt), ('short_name', short_name), ('description', description), ('vl_type', vl_type), ('root_bandwidth', root_bandwidth), ('leaf_bandwidth', leaf_bandwidth), ('cps', cps), ('ip_configuration', ip_configuration), ])


class yc_entity_descriptor_user_entity__entity_descriptor(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /entity_descriptor. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__uuid','__id','__name','__description','__version','__atomic_entities','__virtual_links',)

  _yang_name = 'entity_descriptor'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__uuid = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="uuid", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__description = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__version = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    self.__atomic_entities = YANGDynClass(base=YANGListType("index",yc_atomic_entities_user_entity__entity_descriptor_atomic_entities, yang_name="atomic_entities", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='index', extensions=None), is_container='list', yang_name="atomic_entities", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)
    self.__virtual_links = YANGDynClass(base=YANGListType("id",yc_virtual_links_user_entity__entity_descriptor_virtual_links, yang_name="virtual_links", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='id', extensions=None), is_container='list', yang_name="virtual_links", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return ['entity_descriptor']

  def _get_uuid(self):
    """
    Getter method for uuid, mapped from YANG variable /entity_descriptor/uuid (string)
    """
    return self.__uuid
      
  def _set_uuid(self, v, load=False):
    """
    Setter method for uuid, mapped from YANG variable /entity_descriptor/uuid (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_uuid is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_uuid() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="uuid", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """uuid must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="uuid", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__uuid = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_uuid(self):
    self.__uuid = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="uuid", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_id(self):
    """
    Getter method for id, mapped from YANG variable /entity_descriptor/id (string)
    """
    return self.__id
      
  def _set_id(self, v, load=False):
    """
    Setter method for id, mapped from YANG variable /entity_descriptor/id (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_id is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_id() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """id must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__id = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_id(self):
    self.__id = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="id", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_name(self):
    """
    Getter method for name, mapped from YANG variable /entity_descriptor/name (string)
    """
    return self.__name
      
  def _set_name(self, v, load=False):
    """
    Setter method for name, mapped from YANG variable /entity_descriptor/name (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_name is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_name() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """name must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__name = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_name(self):
    self.__name = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="name", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_description(self):
    """
    Getter method for description, mapped from YANG variable /entity_descriptor/description (string)
    """
    return self.__description
      
  def _set_description(self, v, load=False):
    """
    Setter method for description, mapped from YANG variable /entity_descriptor/description (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_description is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_description() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """description must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__description = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_description(self):
    self.__description = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="description", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_version(self):
    """
    Getter method for version, mapped from YANG variable /entity_descriptor/version (string)
    """
    return self.__version
      
  def _set_version(self, v, load=False):
    """
    Setter method for version, mapped from YANG variable /entity_descriptor/version (string)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_version is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_version() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=six.text_type, is_leaf=True, yang_name="version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """version must be of a type compatible with string""",
          'defined-type': "string",
          'generated-type': """YANGDynClass(base=six.text_type, is_leaf=True, yang_name="version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)""",
        })

    self.__version = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_version(self):
    self.__version = YANGDynClass(base=six.text_type, is_leaf=True, yang_name="version", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='string', is_config=True)


  def _get_atomic_entities(self):
    """
    Getter method for atomic_entities, mapped from YANG variable /entity_descriptor/atomic_entities (list)
    """
    return self.__atomic_entities
      
  def _set_atomic_entities(self, v, load=False):
    """
    Setter method for atomic_entities, mapped from YANG variable /entity_descriptor/atomic_entities (list)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_atomic_entities is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_atomic_entities() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGListType("index",yc_atomic_entities_user_entity__entity_descriptor_atomic_entities, yang_name="atomic_entities", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='index', extensions=None), is_container='list', yang_name="atomic_entities", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """atomic_entities must be of a type compatible with list""",
          'defined-type': "list",
          'generated-type': """YANGDynClass(base=YANGListType("index",yc_atomic_entities_user_entity__entity_descriptor_atomic_entities, yang_name="atomic_entities", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='index', extensions=None), is_container='list', yang_name="atomic_entities", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)""",
        })

    self.__atomic_entities = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_atomic_entities(self):
    self.__atomic_entities = YANGDynClass(base=YANGListType("index",yc_atomic_entities_user_entity__entity_descriptor_atomic_entities, yang_name="atomic_entities", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='index', extensions=None), is_container='list', yang_name="atomic_entities", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)


  def _get_virtual_links(self):
    """
    Getter method for virtual_links, mapped from YANG variable /entity_descriptor/virtual_links (list)
    """
    return self.__virtual_links
      
  def _set_virtual_links(self, v, load=False):
    """
    Setter method for virtual_links, mapped from YANG variable /entity_descriptor/virtual_links (list)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_virtual_links is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_virtual_links() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=YANGListType("id",yc_virtual_links_user_entity__entity_descriptor_virtual_links, yang_name="virtual_links", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='id', extensions=None), is_container='list', yang_name="virtual_links", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """virtual_links must be of a type compatible with list""",
          'defined-type': "list",
          'generated-type': """YANGDynClass(base=YANGListType("id",yc_virtual_links_user_entity__entity_descriptor_virtual_links, yang_name="virtual_links", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='id', extensions=None), is_container='list', yang_name="virtual_links", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)""",
        })

    self.__virtual_links = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_virtual_links(self):
    self.__virtual_links = YANGDynClass(base=YANGListType("id",yc_virtual_links_user_entity__entity_descriptor_virtual_links, yang_name="virtual_links", parent=self, is_container='list', user_ordered=False, path_helper=self._path_helper, yang_keys='id', extensions=None), is_container='list', yang_name="virtual_links", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='list', is_config=True)

  uuid = __builtin__.property(_get_uuid, _set_uuid)
  id = __builtin__.property(_get_id, _set_id)
  name = __builtin__.property(_get_name, _set_name)
  description = __builtin__.property(_get_description, _set_description)
  version = __builtin__.property(_get_version, _set_version)
  atomic_entities = __builtin__.property(_get_atomic_entities, _set_atomic_entities)
  virtual_links = __builtin__.property(_get_virtual_links, _set_virtual_links)


  _pyangbind_elements = OrderedDict([('uuid', uuid), ('id', id), ('name', name), ('description', description), ('version', version), ('atomic_entities', atomic_entities), ('virtual_links', virtual_links), ])


class user_entity(PybindBase):
  """
  This class was auto-generated by the PythonClass plugin for PYANG
  from YANG module user_entity - based on the path /user_entity. Each member element of
  the container is represented as a class variable - with a specific
  YANG type.
  """
  __slots__ = ('_path_helper', '_extmethods', '__entity_descriptor',)

  _yang_name = 'user_entity'

  _pybind_generated_by = 'container'

  def __init__(self, *args, **kwargs):

    self._path_helper = False

    self._extmethods = False
    self.__entity_descriptor = YANGDynClass(base=yc_entity_descriptor_user_entity__entity_descriptor, is_container='container', yang_name="entity_descriptor", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)

    load = kwargs.pop("load", None)
    if args:
      if len(args) > 1:
        raise TypeError("cannot create a YANG container with >1 argument")
      all_attr = True
      for e in self._pyangbind_elements:
        if not hasattr(args[0], e):
          all_attr = False
          break
      if not all_attr:
        raise ValueError("Supplied object did not have the correct attributes")
      for e in self._pyangbind_elements:
        nobj = getattr(args[0], e)
        if nobj._changed() is False:
          continue
        setmethod = getattr(self, "_set_%s" % e)
        if load is None:
          setmethod(getattr(args[0], e))
        else:
          setmethod(getattr(args[0], e), load=load)

  def _path(self):
    if hasattr(self, "_parent"):
      return self._parent._path()+[self._yang_name]
    else:
      return []

  def _get_entity_descriptor(self):
    """
    Getter method for entity_descriptor, mapped from YANG variable /entity_descriptor (container)
    """
    return self.__entity_descriptor
      
  def _set_entity_descriptor(self, v, load=False):
    """
    Setter method for entity_descriptor, mapped from YANG variable /entity_descriptor (container)
    If this variable is read-only (config: false) in the
    source YANG file, then _set_entity_descriptor is considered as a private
    method. Backends looking to populate this variable should
    do so via calling thisObj._set_entity_descriptor() directly.
    """
    if hasattr(v, "_utype"):
      v = v._utype(v)
    try:
      t = YANGDynClass(v,base=yc_entity_descriptor_user_entity__entity_descriptor, is_container='container', yang_name="entity_descriptor", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)
    except (TypeError, ValueError):
      raise ValueError({
          'error-string': """entity_descriptor must be of a type compatible with container""",
          'defined-type': "container",
          'generated-type': """YANGDynClass(base=yc_entity_descriptor_user_entity__entity_descriptor, is_container='container', yang_name="entity_descriptor", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)""",
        })

    self.__entity_descriptor = t
    if hasattr(self, '_set'):
      self._set()

  def _unset_entity_descriptor(self):
    self.__entity_descriptor = YANGDynClass(base=yc_entity_descriptor_user_entity__entity_descriptor, is_container='container', yang_name="entity_descriptor", parent=self, path_helper=self._path_helper, extmethods=self._extmethods, register_paths=True, extensions=None, namespace='urn:fog05:im:user:entity', defining_module='user_entity', yang_type='container', is_config=True)

  entity_descriptor = __builtin__.property(_get_entity_descriptor, _set_entity_descriptor)


  _pyangbind_elements = OrderedDict([('entity_descriptor', entity_descriptor), ])





# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# OCaml implementation and API

import json
from fog05_sdk.interfaces.States import State
from fog05_sdk.types import infra_fdu
from pyangbind.lib.serialise import pybindJSONDecoder
from pyangbind.lib.serialise import pybindJSONEncoder
from collections import OrderedDict
import copy

class InfraFDU(object):

    def __init__(self, data=None):
        '''

        Constructor for the FDU Record

        Parameters
        ----------
        data : dictionary
            optional dictionart containing the FDU record

        returns
        -------
        InfraFDU

        '''
        self.fdu = infra_fdu.infra_fdu()
        self.encoder = pybindJSONEncoder()
        self.uuid = None
        self.fdu_id = None
        self.status = None
        self.image = None
        self.command = None
        self.storage = []
        self.computation_requirements = None
        self.geographical_requirements = None
        self.energy_requirements = None
        self.hypervisor = None
        self.migration_kind = None
        self.configuration = None
        self.interfaces = []
        self.io_ports = []
        self.connection_points = []
        self.depends_on = []
        self.error_code = None
        self.error_msg = None
        self.migration_properties = None
        self.hypervisor_info = None

        if data is not None:
            # data = json.loads(data)
            if isinstance(data['hypervisor_info'], dict):
                data.update({'hypervisor_info':json.dumps(data['hypervisor_info'])})
            pybindJSONDecoder.load_ietf_json({'fdu_record':data}, None, None, obj=self.fdu, skip_unknown=True)
            self.enforce()
            while isinstance(data['hypervisor_info'], str):
                data.update({'hypervisor_info':json.loads(data['hypervisor_info'])})
            self.uuid = self.fdu.fdu_record.uuid
            self.fdu_id = self.fdu.fdu_record.fdu_id
            self.image = data.get('image', None)
            self.command = data.get('command', None)
            self.status = data.get('status')
            self.storage = data.get('storage')
            self.computation_requirements = data.get('computation_requirements')
            self.geographical_requirements = data.get('geographical_requirements', None)
            self.energy_requirements = data.get('energy_requirements', None)
            self.hypervisor = data.get('hypervisor')
            self.migration_kind = data.get('migration_kind')
            self.configuration = data.get('configuration', None)
            self.interfaces = data.get('interfaces')
            self.io_ports = data.get('io_ports')
            self.connection_points = data.get('connection_points')
            self.depends_on = data.get('depends_on')
            self.error_code = data.get('error_code', None)
            self.error_msg = data.get('error_msg', None)
            self.migration_properties = data.get('migration_properties', None)
            self.hypervisor_info = data.get('hypervisor_info')




    def enforce(self):
        if self.fdu.fdu_record.uuid == '':
            raise ValueError('FDU.UUID cannot be empty')

        if self.fdu.fdu_record.fdu_id == '':
            raise ValueError('FDU.FDU_ID cannot be empty')

        if self.fdu.fdu_record.status == '':
            raise ValueError('FDU.Status cannot be empty')

        if self.fdu.fdu_record.computation_requirements.cpu_arch == '':
            raise ValueError('FDU.Computation_Requirements.CPU_Arch cannot be empty')

        if self.fdu.fdu_record.hypervisor == '':
            raise ValueError('FDU.Hypervisor cannot be empty')

        if self.fdu.fdu_record.migration_kind == '':
            raise ValueError('FDU.Migration_Kind cannot be empty')

        if self.fdu.fdu_record.hypervisor_info == '':
            self.fdu.fdu_record.hypervisor_info = '{}'


    def to_json(self):
        '''
        Converts the InfraFDU into a dictionary
        returns
        -------
        dictionary
        '''
        data = {
            'uuid': self.uuid,
            'fdu_id': self.fdu_id,
            'status': self.status,
            'image': self.image,
            'command': self.command,
            'storage': self.storage,
            'computation_requirements': self.computation_requirements,
            'geographical_requirements': self.geographical_requirements,
            'energy_requirements': self.energy_requirements,
            'hypervisor': self.hypervisor,
            'migration_kind': self.migration_kind,
            'configuration': self.configuration,
            'interfaces': self.interfaces,
            'io_ports': self.io_ports,
            'connection_points': self.connection_points,
            'depends_on': self.depends_on,
            'error_code': self.error_code,
            'error_msg': self.error_msg,
            'migration_properties': self.migration_properties,
            'hypervisor_info': self.hypervisor_info
        }
        check_obj = infra_fdu.infra_fdu()
        pybindJSONDecoder.load_ietf_json({'fdu_record':data.update({'hypervisor_info':json.dumps(data['hypervisor_info'])})}, None, None, obj=check_obj, skip_unknown=True)
        data.update({'hypervisor_info':json.loads(data['hypervisor_info'])})
        return data

    def get_short_id(self):
        return ''.join([x[0] for x in self.uuid.split('-')])

    def get_status(self):
        return self.status

    def set_status(self, status) :
        self.status = status

    def get_uuid(self):
        return self.uuid

    def set_uuid(self, uuid) :
        self.uuid = uuid

    def get_fdu_id(self):
        return self.fdu_id

    def set_fdu_id(self, fdu_id) :
        self.fdu_id = fdu_id

    def get_image(self):
        return self.image

    def set_image(self, image) :
        self.image = image

    def get_image_uri(self):
        return self.image.get('uri')

    def set_image_uri(self, uri) :
        self.image.update({'uri':uri})

    def get_image_uuid(self):
        return self.image.get('uuid')

    def set_image_uuid(self, uuid) :
        self.image.update({'uuid':uuid})

    def get_image_format(self):
        return self.image.get('format')

    def set_image_format(self, format) :
        self.image.update({'format':format})

    def get_command(self):
        return self.command

    def set_command(self, command) :
        self.command = command

    def get_storage(self):
        return self.storage

    def set_storage(self, storage) :
        self.storage = storage

    def get_computation_requirements(self):
        return self.computation_requirements

    def set_computation_requirements(self, computation_requirements) :
        self.computation_requirements = computation_requirements

    def get_geographical_requirements(self):
        return self.geographical_requirements

    def set_geographical_requirements(self, geographical_requirements) :
        self.geographical_requirements = geographical_requirements

    def get_energy_requirements(self):
        return self.energy_requirements

    def set_energy_requirements(self, energy_requirements) :
        self.energy_requirements = energy_requirements

    def get_hypervisor(self):
        return self.hypervisor

    def set_hypervisor(self, hypervisor) :
        self.hypervisor = hypervisor

    def get_configuration(self):
        return self.configuration

    def set_configuration(self, configuration) :
        self.configuration = configuration

    def get_io_ports(self):
        return self.io_ports

    def set_io_ports(self, io_ports) :
        self.io_ports = io_ports

    def get_interfaces(self):
        return self.interfaces

    def set_interfaces(self, interfaces) :
        self.interfaces = interfaces

    def get_connection_points(self):
        return self.connection_points

    def set_connection_points(self, connection_points) :
        self.connection_points = connection_points

    def get_depends_on(self):
        return self.depends_on

    def set_depends_on(self, depends_on) :
        self.depends_on = depends_on

    def get_error_code(self):
        return self.error_code

    def set_error_code(self, error_code) :
        self.error_code = error_code

    def get_error_msg(self):
        return self.error_msg

    def set_error_msg(self, error_msg):
        self.error_msg = error_msg

    def get_migration_properties(self):
        return self.migration_properties

    def set_migration_properties(self, source, destination) :
        self.migration_properties = {'source':source, 'destination': destination}

    def clean_migration_properties(self):
        self.migration_properties = None

    def get_hypervisor_info(self):
        return self.hypervisor_info

    def set_hypervisor_info(self, hypervisor_info) :
        self.hypervisor_info = hypervisor_info


    def on_defined(self):
        raise NotImplementedError('This is and interface!')

    def on_configured(self, configuration):
        raise NotImplementedError('This is and interface!')

    def on_clean(self):
        raise NotImplementedError('This is and interface!')

    def on_start(self):
        raise NotImplementedError('This is and interface!')

    def on_stop(self):
        raise NotImplementedError('This is and interface!')

    def on_pause(self):
        raise NotImplementedError('This is and interface!')

    def on_resume(self):
        raise NotImplementedError('This is and interface!')

    def before_migrate(self):
        raise NotImplementedError('This is and interface!')

    def after_migrate(self):
        raise NotImplementedError('This is and interface!')

    def __str__(self):
        return "FDU Record UUID: {} FDU_ID: {}".format(self.uuid, self.fdu_id)



# Copyright (c) 2014,2018 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# Initial implementation and API


from enum import Enum


class State(Enum):

    """
    States of entities
    """

    ERROR = 'ERROR'
    UNDEFINED = 'UNDEFINE'
    DEFINED = 'DEFINE'
    CONFIGURED = 'CONFIGURE'
    RUNNING = 'RUN'
    PAUSED = 'PAUSE'
    SCALING = 'SCALE'
    MIGRATING = 'MIGRATE'
    # Migration concurrent states
    TAKING_OFF = 'TAKE_OFF'
    LANDING = 'LAND'



# Copyright (c) 2014,2019 Contributors to the Eclipse Foundation
#
# See the NOTICE file(s) distributed with this work for additional
# information regarding copyright ownership.
#
# This program and the accompanying materials are made available under the
# terms of the Eclipse Public License 2.0 which is available at
# http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
# which is available at https://www.apache.org/licenses/LICENSE-2.0.
#
# SPDX-License-Identifier: EPL-2.0 OR Apache-2.0
#
# Contributors: Gabriele Baldoni, ADLINK Technology Inc.
# Entity Parsers and Python Class


import json
from fog05_sdk.types import infra_entity
from pyangbind.lib.serialise import pybindJSONDecoder
from pyangbind.lib.serialise import pybindJSONEncoder
from collections import OrderedDict


class EntityRecord(object):

    def __init__(self, data=None):
        '''

        Constructor for the FDU Descriptor
        :param data dictionary containing the FDU descriptor

        :return the FDU object

        '''

        self.e = infra_entity.infra_entity()
        self.encoder = pybindJSONEncoder()
        self.entity_id = None
        self.uuid = None
        self.atomic_entities = []
        self.virtual_links = []
        if data is not None:
            pybindJSONDecoder.load_ietf_json({'entity_record':data}, None, None, obj=self.e, skip_unknown=True)
            self.enforce()

            self.entity_id = self.e.entity_record.entity_id
            self.uuid =  self.e.entity_record.uuid
            self.atomic_entities = data.get('atomic_entities')
            self.virtual_links = data.get('virtual_links')


    def enforce(self):
        if self.e.entity_record.entity_id == '':
            raise ValueError('EntityRecord.ID cannot be empty')

        if self.e.entity_record.uuid == '':
            raise ValueError('EntityRecord.UUID cannot be empty')

        if len (self.e.entity_record.atomic_entities) == 0:
            raise ValueError('EntityRecord.atomic_entities cannot be empty')

    def to_json(self):
        data = {
            'uuid': self.uuid,
            'entity_id': self.entity_id,
            'atomic_entities': self.atomic_entities,
            'virtual_links': self.virtual_links
        }
        check_obj = infra_entity.infra_entity()
        pybindJSONDecoder.load_ietf_json({'entity_record':data}, None, None, obj=check_obj, skip_unknown=True)
        return data

    def get_uuid(self):
        return self.uuid

    def set_uuid(self, uuid) :
        self.uuid = uuid

    def get_entity_id(self):
        return self.entity_id

    def set_entity_id(self, entity_id) :
        self.entity_id = entity_id

    def get_connection_points(self):
        return self.connection_points

    def set_connection_points(self, connection_points) :
        self.connection_points = connection_points

    def set_virtual_links(self, virtual_links):
        self.virtual_links = virtual_links

    def get_virtual_links(self):
        return self.virtual_links

    def set_atomic_entities(self, atomic_entities):
        self.atomic_entities = atomic_entities

    def get_atomic_entities(self):
        return self.atomic_entities

    def __str__(self):
        return "UUID : {} Entity ID: {}".format(self.uuid, self.entity_id)



from enum import Enum


aroot = '/afos'
droot = '/dfos'
sroot = '/sfos'


global_actual_prefix = '/agfos'
global_desired_prefix = '/dgfos'
local_actual_prefix = '/alfos'
local_desired_prefix = '/dlfos'

local_constraint_actual_prefix = '/aclfos'
local_constaint_desired_prefix = '/dclfos'

default_system_id = '0'
default_tenant_id = '0'




class FDUState(Enum):

    '''
    States of FDUs
    '''
    DEFINE = 'DEFINE'
    CONFIGURE = 'CONFIGURE'
    CLEAN = 'CLEAN'
    RUN = 'RUN'
    STARTING = 'STARTING'
    STOP = 'STOP'
    RESUME = 'RESUME'
    PAUSE = 'PAUSE'
    SCALE = 'SCALE'
    TAKE_OFF = 'TAKE_OFF'
    LAND = 'LAND'
    MIGRATE = 'MIGRATE'
    UNDEFINE = 'UNDEFINE'
    ERROR = 'ERROR'


class ErrorCodes(Enum):
    '''
    FDUs error codes
    '''

    DEFINE_ERROR = 1
    CONFIGURE_ERROR = 2
    CLEAN_ERROR = 3
    RUN_ERROR = 4
    STOP_ERROR = 5
    PAUSE_ERROR = 6
    TAKE_OFF_ERROR = 8
    LAND_ERROR = 9
    MIGRATE_ERROR = 10
    UNDEFINE_ERROR = 0

    FDU_DESCRIPTOR_NOT_FOUND = 11
    NETWORK_DESCRIPTOR_NOT_FOUND = 22
    CONNECTION_POINT_DESCRIPTOR_NOT_FOUND = 33

    IMAGE_NOT_FOUND = 21
    COMPUTE_REQUIREMENTS_ERROR = 23
    HYPERVISOR_ERROR = 41








def create_path(tokens):
    return '/'.join(tokens)


def append_to_path(base, extention):
    return '{}/{}'.format(base, extention)
